<?php
//
// VPNKI connections install script. Version 1.1.
// Install pptp-linux, xl2tpd, strongswan, openvpn.
// Tested on Raspbian GNU/Linux 8 (jessie) 
// 20-01-2019
//
//
ini_set("log_errors", 1);
ini_set("error_log", "/var/log/vpnki_install.log");
error_log( "### Start VPNKI INSTALL ###" );

// Color class
	class Colors {
		private $foreground_colors = array();
		private $background_colors = array();

		public function __construct() {
			// Set up shell colors
			$this->foreground_colors['black'] = '0;30';
			$this->foreground_colors['dark_gray'] = '1;30';
			$this->foreground_colors['blue'] = '0;34';
			$this->foreground_colors['light_blue'] = '1;34';
			$this->foreground_colors['green'] = '0;32';
			$this->foreground_colors['light_green'] = '1;32';
			$this->foreground_colors['cyan'] = '0;36';
			$this->foreground_colors['light_cyan'] = '1;36';
			$this->foreground_colors['red'] = '0;31';
			$this->foreground_colors['light_red'] = '1;31';
			$this->foreground_colors['purple'] = '0;35';
			$this->foreground_colors['light_purple'] = '1;35';
			$this->foreground_colors['brown'] = '0;33';
			$this->foreground_colors['yellow'] = '1;33';
			$this->foreground_colors['light_gray'] = '0;37';
			$this->foreground_colors['white'] = '1;37';

			$this->background_colors['black'] = '40';
			$this->background_colors['red'] = '41';
			$this->background_colors['green'] = '42';
			$this->background_colors['yellow'] = '43';
			$this->background_colors['blue'] = '44';
			$this->background_colors['magenta'] = '45';
			$this->background_colors['cyan'] = '46';
			$this->background_colors['light_gray'] = '47';
		}

		// Returns colored string
		public function getColoredString($string, $foreground_color = null, $background_color = null) {
			$colored_string = "";

			// Check if given foreground color found
			if (isset($this->foreground_colors[$foreground_color])) {
				$colored_string .= "\033[" . $this->foreground_colors[$foreground_color] . "m";
			}
			// Check if given background color found
			if (isset($this->background_colors[$background_color])) {
				$colored_string .= "\033[" . $this->background_colors[$background_color] . "m";
			}

			// Add string and end coloring
			$colored_string .=  $string . "\033[0m";

			return $colored_string;
		}

		// Returns all foreground color names
		public function getForegroundColors() {
			return array_keys($this->foreground_colors);
		}

		// Returns all background color names
		public function getBackgroundColors() {
			return array_keys($this->background_colors);
		}
	}


$colors = new Colors();

echo "\n";
echo $colors->getColoredString("===============================================", "cyan") . "\n";
echo $colors->getColoredString("   Welcome to vpnki connection installation!", "cyan") . "\n";
echo $colors->getColoredString("===============================================", "cyan") . "\n\n";
echo "   This script install:\n\n";
echo "   - pptp-linux package for PPTP connection\n";
echo "   - xl2tpd package for L2TP connection\n";
echo "   - strongswan package for IPsec connection\n";
echo "   - openvpn package for OpenVPN connection.\n\n";

// Check OS version
echo "\n";
echo "   Checking OS version...\n";
error_log( "Checking OS version......" );

$version=shell_exec('cat /etc/os-release');

if (strpos($version, 'Raspbian') || strpos($version,'Debian') || strpos($version,'Ubuntu') !== false) {
        echo "   Correct version\n\n";
	error_log( "Correct version......" );
}
else{

        echo $colors->getColoredString("\n   WARNING! Wrong OS version. Aborting...", "red") . "\n\n";
	error_log( "WARNING! Wrong OS version. Aborting..." );
        exit(1);
}



echo "   Do you have API-key from vpnki.ru?\n";
echo "   Type 'yes' or 'no' or 'Ctrl+C' to abort: \n";
echo "   >";
$handle = fopen ("php://stdin","r");
$line = fgets($handle);
// Checking API key validation
if(trim($line) == 'yes'){
	echo "   Please enter API-key:\n";
	echo "   >";
	$handle1 = fopen ("php://stdin","r");
	$key = fgets($handle1);
	$key = str_replace("\n", "", $key);
	$key = str_replace("\r", "", $key);

        echo "   Checking for curl...\n\n";
	$curl=shell_exec('apt-get install -y curl');
	echo "   Checking API key validation...\n\n";
	$url="\"https://vpnki.ru/index.php?option=com_api&format=raw&app=webservices&resource=tunnels&key=$key\"";

	$string = "curl -s $url";
	$key_valid=shell_exec($string);

	$object=json_decode($key_valid);
	$array=json_decode($key_valid, true);
//      var_dump(json_decode($key_valid));

	$length=sizeof($object);

	$code = $object->code;

		if($code == '403'){
		        echo $colors->getColoredString("   WARNING! Wrong API key. Continue without API-key", "red") . "\n";
			$flag_key="1";
		}

		else {
			echo "   Correct API key. Continue...\n";
			$flag_key="0";
		}
	fclose($handle1);
        fclose($handle);
}
else {
	echo $colors->getColoredString("   WARNING! Continue without API-key", "red") . "\n";
	$flag_key="1";
	fclose($handle1);
	fclose($handle);

}


// -----------------------------------------------------------
// Start PPTP installation
// -----------------------------------------------------------
echo "\n";
echo $colors->getColoredString("   1. Install PPTP...", "cyan") . "\n";
echo "   Checking requirements for PPTP...";
sleep (1);
echo "done\n";

// Start PPTP-LINUX installation
echo "   Start to install pptp-linux... Please wait...\n";
error_log( "Start install pptp-linux package" );
$pptp=shell_exec('apt-get install -y pptp-linux');
echo "   pptp-linux installed\n\n";
error_log( "Installed" );

// Configure PPTP-LINUX
echo "   Configure pptp-linux...\n";
echo $colors->getColoredString("   Creating connection to msk.vpnki.ru", "yellow") . "\n";
error_log( "Creating connection to msk.vpnki.ru" );

// Show all tunnels from vpnki.ru
if ($flag_key == "0"){

        if ($array[0]['username'] == NULL){
                echo "   You haven't any tunnels configured on vpnki.ru\n";
        }
        else {
                echo "   You have " .$length ." tunnel entries at vpnki.ru:\n";

                for ($i = 0; $i < $length; $i++) {
                        echo "   " . $array[$i]['username'] . "\n";
                }

        }
}
else {
}

echo "   Please enter username for VPN tunnel (example - user761283) :\n";
echo "   >";
$handle1 = fopen ("php://stdin","r");
$username_line = fgets($handle1);

echo "   Please enter password:\n";
echo "   >";
$handle2 = fopen ("php://stdin","r");
$password_line = fgets($handle2);

$username_line = str_replace("\n", "", $username_line);
$username_line = str_replace("\r", "", $username_line);
$password_line = str_replace("\n", "", $password_line);
$password_line = str_replace("\r", "", $password_line);

$install="sudo pptpsetup --create vpnki --server msk.vpnki.ru --username " . $username_line . " --password " . $password_line;
$run=shell_exec($install);
echo $run;

echo $colors->getColoredString("   Connection to vpnki with username " . $username_line . " and password " . $password_line . " created successfully", "yellow") . "\n\n";
sleep(1);
//echo "   Lets configure /etc/rc.local...\n";

//$text = <<<TEXT

//# ======================
//#  Added by vpnki setup
//# ----------------------
//printf "VPN connection to VPNKI"
//pon vpnki updetach
//sleep 5
//sudo route add -net "172.16.0.0/16" dev "ppp0" #Маршрут к сети VPNKI
//#sudo route add -net "192.168.100.0/24" dev "ppp0" #Например маршрут к "другой" части вашей сети (в сеть 192.168.100.0/24)
//printf "Netstat output of all PPTP sockets"
//netstat -a | grep "/var/run/pptp/"
//# =====================

//exit 0
//TEXT;

// Replace last match of a string
//function lreplace($search, $replace, $subject){
//    $pos = strrpos($subject, $search);
//    if($pos !== false){
//        $subject = substr_replace($subject, $replace, $pos, strlen($search));
//    }
//    return $subject;
//}

//$str=file_get_contents('/etc/rc.local');
//$oldMessage = "exit 0";

//$str = lreplace($oldMessage, $text, $str);

//$myfile = fopen("/etc/rc.local", "wr") or die("Unable to open file!");
//$myfile = file_put_contents('/etc/rc.local', $str.PHP_EOL , FILE_APPEND | LOCK_EX);
//fclose($myfile);
//echo "   Configuration done\n\n";

echo $colors->getColoredString("   Configure session params in /etc/ppp/peers/vpnki...", "yellow") . "\n";
$text = <<<TEXT
persist
maxfail 0
holdoff 10
TEXT;

$myfile = fopen("/etc/ppp/peers/vpnki", "a") or die("Unable to open file /etc/ppp/peers/vpnki!");
$myfile = file_put_contents('/etc/ppp/peers/vpnki', $text.PHP_EOL , FILE_APPEND | LOCK_EX);
echo "   Configuration done\n\n";

echo $colors->getColoredString("   Configure route to vpnki /etc/ppp/ip-up/routeadd...", "yellow") . "\n"; 
$text = <<<TEXT
#!/bin/sh -e
route add -net "172.16.0.0/16" dev "ppp0" #Маршрут к сети VPNKI
#route add -net "192.168.100.0/24" dev "ppp0" #Например маршрут к вашей "другой" сети 192.168.100.0/24 (если такая существует через сеть VPNKI)
TEXT;

shell_exec('touch /etc/ppp/ip-up.d/routeadd');
$myfile = fopen("/etc/ppp/ip-up.d/routeadd", "a") or die("Unable to open file /etc/ppp/ip-up.d/routeadd!");
$myfile = file_put_contents('/etc/ppp/ip-up.d/routeadd', $text.PHP_EOL , FILE_APPEND | LOCK_EX);

shell_exec('chmod 755 /etc/ppp/ip-up.d/routeadd');
echo "   Configuration done\n\n";

echo "   PPTP installed successfully\n\n";
error_log( "PPTP installed successfully" );


// ----------------------------------------------------------
// Start L2TP installation
// ----------------------------------------------------------
echo "\n";
echo $colors->getColoredString("   2. Install L2TP...", "cyan") . "\n";
echo "   Checking requirements for L2TP...";
//sleep 1;
echo "done\n";
echo "   Start install xl2tpd... Please wait...\n";
error_log( "Start install xl2tpd" );
$xl2tpd=shell_exec('apt-get install -y xl2tpd');
echo "   xl2tpd installed\n";
error_log( "Installed" );

// Configure xl2tpd
echo "   Configure xl2tpd...\n\n";
echo $colors->getColoredString("   Configure LAC /etc/xl2tpd/xl2tpd.conf...", "yellow") . "\n";

$text = <<<TEXT
[lac vpnki]
lns = msk.vpnki.ru
require chap = yes
refuse pap = yes
require authentication = yes
name = $username_line
ppp debug = yes
pppoptfile = /etc/ppp/options.l2tpd.client
length bit = yes
TEXT;

$myfile = fopen("/etc/xl2tpd/xl2tpd.conf", "a") or die("Unable to open file /etc/xl2tpd/xl2tpd.conf!");
$myfile = file_put_contents('/etc/xl2tpd/xl2tpd.conf', $text.PHP_EOL , FILE_APPEND | LOCK_EX);
echo "   Configuration done\n\n";

echo $colors->getColoredString("   Configure session params /etc/ppp/options.l2tpd.client...", "yellow") . "\n";;
error_log( "Configure params..." );

$text = <<<TEXT
ipcp-accept-local
ipcp-accept-remote
refuse-eap
require-chap
noccp
noauth
#crtscts # - возможно нужно закомментировать, если в /var/log/syslog есть запись об ошибке с этой строкой
idle 1800
mtu 1410
mru 1410
nodefaultroute
debug
lock
#proxyarp
connect-delay 5000
name $username_line
password $password_line
remotename vpnki
ipparam vpnki
TEXT;

shell_exec('touch /etc/ppp/options.l2tpd.client');
$myfile = fopen("/etc/ppp/options.l2tpd.client", "a") or die("Unable to open file /etc/ppp/options.l2tpd.client!");
$myfile = file_put_contents('/etc/ppp/options.l2tpd.client', $text.PHP_EOL , FILE_APPEND | LOCK_EX);
echo "   Configuration done\n\n";

echo $colors->getColoredString("   Configure control file /var/run/xl2tpd/l2tp-control...", "yellow") . "\n";
shell_exec('mkdir -p /var/run/xl2tpd');
shell_exec('touch /var/run/xl2tpd/l2tp-control');
shell_exec('service xl2tpd restart');
echo "   Configuration done\n\n";
echo "   L2TP installed successfully\n\n";
error_log( "L2TP installed successfully" );

// ----------------------------------------------------------
// Start IPsec installation
// ----------------------------------------------------------
echo "\n";
echo $colors->getColoredString("   3. Install IPsec...", "cyan") . "\n";
echo "   Checking requirements for strongswan...";
//sleep 1;
echo "done\n";
echo "   Start to install strongswan... Please wait...\n";
error_log( "Start to install strongswan" );
$strongswan=shell_exec('apt-get install -y strongswan');
echo "   Strongswan installed\n\n";
error_log( "Installed" );

// Configure strongswan
echo $colors->getColoredString("   Write shared key in /etc/ipsec.secrets...", "yellow") . "\n";

$text = <<<TEXT
: PSK "vpnki"
TEXT;

$myfile = fopen("/etc/ipsec.secrets", "a") or die("Unable to open file /etc/ipsec.secrets!");
$myfile = file_put_contents('/etc/ipsec.secrets', $text.PHP_EOL , FILE_APPEND | LOCK_EX);
echo "   Configuration done\n\n";

echo $colors->getColoredString("   Configure IPsec connection in /etc/ipsec.conf...", "yellow") . "\n";
$text = <<<TEXT
conn %default
   ikelifetime=16h
   keylife=12h
   rekeymargin=3m
   keyingtries=1
   keyexchange=ikev1
   authby=secret
   aggressive = no

conn vpnki-l2tp
   authby=secret
   auto=add
   rekey=no
   type=transport
   right=msk.vpnki.ru
   rightid=%any
   rightprotoport=17/1701
   ike=aes256-sha1-modp2048
   esp=aes256-null
   left=%any
   leftprotoport=17/%any
   dpdaction=clear
TEXT;

$myfile = fopen("/etc/ipsec.conf", "a") or die("Unable to open file /etc/ipsec.conf!");
$myfile = file_put_contents('/etc/ipsec.conf', $text.PHP_EOL , FILE_APPEND | LOCK_EX);
echo "   Configuration done\n\n";
echo "   IPsec installed successfully\n\n";
error_log( "IPsec installed successfully" );

// ----------------------------------------------------------
// Start OpenVPN installation
// ----------------------------------------------------------
echo "\n";
echo $colors->getColoredString("   4. Install OpenVPN...", "cyan") . "\n";
echo "   Checking requirements for OpenVPN...";
//sleep 1;
echo "done\n";
echo "   Install OpenVPN... Please wait...\n";
error_log( "Start install OpenVPN" );
$openvpn=shell_exec('apt-get install -y openvpn');
echo "   OpenVPN installed\n";
error_log( "Installed" );

// Configure openvpn
echo $colors->getColoredString("   Configure openvpn...", "yellow") . "\n";

// If user has API key then
if ($flag_key == "0"){
	echo "   Download .ovpn file...\n\n";
	$url="\"https://vpnki.ru/index.php?option=com_api&format=raw&app=webservices&resource=ovpn&action=add&key=$key\"";

	$string = "curl -s $url";
	$ovpn=shell_exec($string);
	$object=json_decode($ovpn);
	$array=json_decode($ovpn, true);
//        var_dump(json_decode($ovpn));

//	$filename=$array[0]['filename'];
//	$file_content=$array[0]['file_content'];
//	$filename=$object->filename;
	$file_content=$object->file_content;
	$myfile = fopen("/etc/openvpn/vpnki.conf", "wr") or die("Unable to open file /etc/openvpn/vpnki.conf...!");
	$myfile = file_put_contents("/etc/openvpn/vpnki.conf", $file_content.PHP_EOL , FILE_APPEND | LOCK_EX);
        echo "   Configuration done\n\n";

$text = <<<TEXT
$username_line
$password_line
TEXT;

	echo $colors->getColoredString("   Configure login and pass in /etc/openvpn/vpnki_login...", "yellow") . "\n";
	$myfile = fopen("/etc/openvpn/vpnki_login", "wr") or die("Unable to open file /etc/openvpn/vpnki_login!");
	$myfile = file_put_contents('/etc/openvpn/vpnki_login', $text.PHP_EOL , FILE_APPEND | LOCK_EX);
	echo "   Configuration done\n\n";

        echo $colors->getColoredString("   Configure link to /etc/openvpn/vpnki_login in /etc/openvpn/vpnki.conf...", "yellow") . "\n";

$text = <<<TEXT
auth-user-pass /etc/openvpn/vpnki_login
<ca>
TEXT;

// Replace last match of a string
	function lreplace($search, $replace, $subject){
    	$pos = strrpos($subject, $search);
	    if($pos !== false){
	        $subject = substr_replace($subject, $replace, $pos, strlen($search));
	    }
	    return $subject;
	}

	$str=file_get_contents('/etc/openvpn/vpnki.conf');
	$oldMessage = "<ca>";

	$str = lreplace($oldMessage, $text, $str);

	$myfile = fopen("/etc/openvpn/vpnki.conf", "wr") or die("Unable to open file!");
	$myfile = file_put_contents('/etc/openvpn/vpnki.conf', $str.PHP_EOL , FILE_APPEND | LOCK_EX);
	echo "   Configuration done\n\n";

}
else {
        echo "   API key is not entered. Please start OpenVPN on vpnki.ru page and download .ovpn file manually.\n\n";
}

echo "   OpenVPN installed successfully\n\n";
error_log( "OpenVPN installed successfully" );


echo "\n";
echo $colors->getColoredString("===============================================", "cyan") . "\n";
echo $colors->getColoredString("   Congrats! vpnki installed successfully!", "cyan") . "\n";
echo $colors->getColoredString("===============================================", "cyan") . "\n\n";

error_log( "### Stop VPNKI INSTALL ###".PHP_EOL );
exit;
?>
